# encoding: utf-8
"""

Copyright (c) 2016 Job Snijders <job@ntt.net>
Copyright (c) 2009-2017 Exa Networks. All rights reserved.
License: 3-clause BSD. (See the COPYRIGHT file)
"""

# Large Community should be imported from this file
# as it makes sure that all the registering decorator are run

from exabgp.bgp.message.update.attribute.community.large.community import LargeCommunity
from exabgp.bgp.message.update.attribute.community.large.communities import LargeCommunities
