# encoding: utf-8
"""
community.py

Created by Thomas Mangin on 2009-11-05.
Copyright (c) 2009-2017 Exa Networks. All rights reserved.
License: 3-clause BSD. (See the COPYRIGHT file)
"""

from exabgp.bgp.message.update.attribute.community.initial import Community
from exabgp.bgp.message.update.attribute.community.initial import Communities

from exabgp.bgp.message.update.attribute.community.large import LargeCommunity
from exabgp.bgp.message.update.attribute.community.large import LargeCommunities

from exabgp.bgp.message.update.attribute.community.extended import ExtendedCommunity
from exabgp.bgp.message.update.attribute.community.extended import ExtendedCommunities
